@@ -4,12 +4,12 @@
 // MS compatible compilers support #pragma once
 #if defined(_MSC_VER) && (_MSC_VER >= 1020)
 # pragma once
-#pragma inline_depth(255)
+#pragma inline_depth(511)
 #pragma inline_recursion(on)
 #endif
 
 #if defined(__MWERKS__)
-#pragma inline_depth(255)
+#pragma inline_depth(511)
 #endif
 
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
@@ -24,28 +24,28 @@
 
 #include <new>     // for placement new
 #include <memory>  // for auto_ptr
-#include <cstddef> // size_t
+#include <cstddef> // size_t, NULL
 
 #include <boost/config.hpp>
+#include <boost/detail/workaround.hpp>
 #if defined(BOOST_NO_STDC_NAMESPACE)
 namespace std{ 
     using ::size_t; 
 } // namespace std
 #endif
-
-#include <boost/detail/workaround.hpp>
-
+#include <boost/serialization/throw_exception.hpp>
+#include <boost/serialization/smart_cast.hpp>
 #include <boost/static_assert.hpp>
-#include <boost/static_warning.hpp>
-#include <boost/smart_cast.hpp>
+#include <boost/serialization/static_warning.hpp>
+#include <boost/detail/no_exceptions_support.hpp>
 
 #include <boost/type_traits/is_pointer.hpp>
-#include <boost/type_traits/is_fundamental.hpp>
 #include <boost/type_traits/is_enum.hpp>
-#include <boost/type_traits/is_same.hpp>
+#include <boost/type_traits/is_const.hpp>
 #include <boost/type_traits/remove_const.hpp>
-#include <boost/throw_exception.hpp>
-#include <boost/serialization/is_abstract.hpp>
+#include <boost/type_traits/remove_extent.hpp>
+#include <boost/serialization/assume_abstract.hpp>
+#include <boost/type_traits/is_polymorphic.hpp>
 
 #include <boost/mpl/eval_if.hpp>
 #include <boost/mpl/if.hpp>
@@ -58,32 +58,34 @@ namespace std{
 #include <boost/mpl/list.hpp>
 #include <boost/mpl/empty.hpp>
 #include <boost/mpl/not.hpp>
-// the following is only used with VC for now and it crashes
-// at least one other compiler (Borland)
-#if defined(BOOST_MSVC)
-#include <boost/mpl/find.hpp>
-#endif
+#include <boost/mpl/bool.hpp>
 
+ #ifndef BOOST_SERIALIZATION_DEFAULT_TYPE_INFO   
+     #include <boost/serialization/extended_type_info_typeid.hpp>   
+ #endif 
 // the following is need only for dynamic cast of polymorphic pointers
 #include <boost/archive/detail/basic_iarchive.hpp>
 #include <boost/archive/detail/basic_iserializer.hpp>
 #include <boost/archive/detail/archive_pointer_iserializer.hpp>
+#include <boost/archive/archive_exception.hpp>
 
-#include <boost/serialization/force_include.hpp>
-#include <boost/serialization/extended_type_info.hpp>
 #include <boost/serialization/serialization.hpp>
 #include <boost/serialization/version.hpp>
 #include <boost/serialization/level.hpp>
 #include <boost/serialization/tracking.hpp>
 #include <boost/serialization/type_info_implementation.hpp>
 #include <boost/serialization/nvp.hpp>
-#include <boost/serialization/force_include.hpp>
-
-#include <boost/archive/archive_exception.hpp>
-
-#include <boost/archive/detail/known_archive_types_fwd.hpp>
+#include <boost/serialization/void_cast.hpp>
+#include <boost/serialization/array.hpp>
+#include <boost/serialization/collection_size_type.hpp>
+#include <boost/serialization/singleton.hpp>
 
 namespace boost {
+
+namespace serialization {
+    class extended_type_info;
+} // namespace serialization
+
 namespace archive {
 
 // an accessor to permit friend access to archives.  Needed because
@@ -105,41 +107,36 @@ private:
     virtual void destroy(/*const*/ void *address) const {
         boost::serialization::access::destroy(static_cast<T *>(address));
     }
+    // private constructor to inhibit any existence other than the 
+    // static one
 public:
     explicit iserializer() :
         basic_iserializer(
-            * boost::serialization::type_info_implementation<T>::type::get_instance()
+            boost::serialization::type_info_implementation<T>::type
+                ::get_const_instance()
         )
     {}
     virtual BOOST_DLLEXPORT void load_object_data(
         basic_iarchive & ar,
         void *x, 
         const unsigned int file_version
-    ) const BOOST_USED ;
+    ) const BOOST_USED;
     virtual bool class_info() const {
         return boost::serialization::implementation_level<T>::value 
             >= boost::serialization::object_class_info;
     }
-    virtual bool tracking() const {
+    virtual bool tracking(const unsigned int /* flags */) const {
         return boost::serialization::tracking_level<T>::value 
                 == boost::serialization::track_always
-            || boost::serialization::tracking_level<T>::value 
-                == boost::serialization::track_selectivly
-            && serialized_as_pointer();
+            || ( boost::serialization::tracking_level<T>::value 
+                == boost::serialization::track_selectively
+                && serialized_as_pointer());
     }
     virtual unsigned int version() const {
         return ::boost::serialization::version<T>::value;
     }
     virtual bool is_polymorphic() const {
-        typedef BOOST_DEDUCED_TYPENAME 
-            boost::serialization::type_info_implementation<
-                T
-            >::type::is_polymorphic::type typex;
-        return typex::value;
-    }
-    static iserializer & instantiate(){
-        static iserializer instance;
-        return instance;
+        return boost::is_polymorphic<T>::value;
     }
     virtual ~iserializer(){};
 };
@@ -152,49 +149,32 @@ BOOST_DLLEXPORT void iserializer<Archive, T>::load_object_data(
 ) const {
     // make sure call is routed through the higest interface that might
     // be specialized by the user.
-    boost::serialization::serialize_adl<Archive, T>(
-        boost::smart_cast_reference<Archive &>(ar),
+    boost::serialization::serialize_adl(
+        boost::serialization::smart_cast_reference<Archive &>(ar),
         * static_cast<T *>(x), 
         file_version
     );
 }
 
-// instantiation of this template creates a static object.  Note inversion of
-// normal argument order to workaround bizarre error in MSVC 6.0 which only
-// manifests iftself during compiler time.
-template<class T, class Archive>
-class pointer_iserializer : public archive_pointer_iserializer<Archive> 
+template<class Archive, class T>
+class pointer_iserializer
+  : public archive_pointer_iserializer<Archive>
 {
 private:
     virtual const basic_iserializer & get_basic_serializer() const {
-        return iserializer<Archive, T>::instantiate();
+        return boost::serialization::singleton<
+            iserializer<Archive, T>
+        >::get_const_instance();
     }
-    virtual BOOST_DLLEXPORT void load_object_ptr(
+    BOOST_DLLEXPORT virtual void load_object_ptr(
         basic_iarchive & ar, 
         void * & x,
         const unsigned int file_version
     ) const BOOST_USED;
 public:
-    static const pointer_iserializer instance;
-    explicit pointer_iserializer() :
-        archive_pointer_iserializer<Archive>(
-            * boost::serialization::type_info_implementation<T>::type::get_instance()
-        )
-    {
-        basic_iserializer & bis = iserializer<Archive, T>::instantiate();
-        bis.set_bpis(this);
-    }
-    static BOOST_DLLEXPORT const pointer_iserializer & instantiate() BOOST_USED;
-    virtual ~pointer_iserializer(){};
+    pointer_iserializer();
 };
 
-// note: instances of this template to be constructed before the main
-// is called in order for things to be initialized properly.  For this
-// reason, hiding the instance in a static function as was done above
-// won't work here so we created a free instance here.
-template<class T, class Archive>
-const pointer_iserializer<T, Archive> pointer_iserializer<T, Archive>::instance;
-
 // note trick to be sure that operator new is using class specific
 // version if such exists. Due to Peter Dimov.
 // note: the following fails if T has no default constructor.
@@ -266,39 +246,57 @@ private:
 };
 
 template<class Archive, class T>
-void load_ptr(
-    Archive & ar, 
-    T * & t, 
-    const BOOST_PFTO unsigned int file_version
-){
-}
-
-template<class T, class Archive>
-BOOST_DLLEXPORT void pointer_iserializer<T, Archive>::load_object_ptr(
+BOOST_DLLEXPORT void pointer_iserializer<Archive, T>::load_object_ptr(
     basic_iarchive & ar, 
     void * & x,
     const unsigned int file_version
-) const {
+) const
+{
+    Archive & ar_impl = 
+        boost::serialization::smart_cast_reference<Archive &>(ar);
+
     auto_ptr_with_deleter<T> ap(heap_allocator<T>::invoke());
     if(NULL == ap.get())
-        boost::throw_exception(std::bad_alloc()) ;
+        boost::serialization::throw_exception(std::bad_alloc()) ;
+
     T * t = ap.get();
     x = t;
 
-    // this addresses an obscure situtation that occurs when load_constructor
-    // de-serializes something through and a pointer.
-    ar.next_object_pointer(t);
+    // catch exception during load_construct_data so that we don't
+    // automatically delete the t which is most likely not fully
+    // constructed
+    BOOST_TRY {
+        // this addresses an obscure situtation that occurs when 
+        // load_constructor de-serializes something through a pointer.
+        ar.next_object_pointer(t);
+        boost::serialization::load_construct_data_adl<Archive, T>(
+            ar_impl,
+            t, 
+            file_version
+        );
+    }
+    BOOST_CATCH(...){
+        ap.release();
+        BOOST_RETHROW;
+    }
+    BOOST_CATCH_END
 
-    Archive & ar_impl = boost::smart_cast_reference<Archive &>(ar);
-    boost::serialization::load_construct_data_adl<Archive, T>(
-        ar_impl,
-        t, 
-        file_version
-    );
     ar_impl >> boost::serialization::make_nvp(NULL, * t);
     ap.release();
 }
 
+template<class Archive, class T>
+pointer_iserializer<Archive, T>::pointer_iserializer() :
+    archive_pointer_iserializer<Archive>(
+        boost::serialization::type_info_implementation<T>::type
+            ::get_const_instance()
+    )
+{
+    boost::serialization::singleton<
+        iserializer<Archive, T>
+    >::get_mutable_instance().set_bpis(this);
+}
+
 template<class Archive, class T>
 struct load_non_pointer_type {
     // note this bounces the call right back to the archive
@@ -320,49 +318,76 @@ struct load_non_pointer_type {
             );
         }
     };
+
     // note this save class information including version
     // and serialization level to the archive
-    struct load {
+    struct load_standard {
         static void invoke(Archive &ar, T &t){
-            const void * x = &t;
-            ar.load_object(const_cast<void *>(x), iserializer<Archive, T>::instantiate());
+            //BOOST_STATIC_ASSERT(! boost::is_const<T>::value);
+            // borland - for some reason T is const here - even though
+            // its not called that way - so fix it her
+            typedef BOOST_DEDUCED_TYPENAME boost::remove_const<T>::type typex;
+            void * x = & const_cast<typex &>(t);
+            ar.load_object(
+                x, 
+                boost::serialization::singleton<
+                    iserializer<Archive, T>
+                >::get_const_instance()
+            );
         }
     };
+
+    struct load_conditional {
+        static void invoke(Archive &ar, T &t){
+            //if(0 == (ar.get_flags() & no_tracking))
+                load_standard::invoke(ar, t);
+            //else
+            //    load_only::invoke(ar, t);
+        }
+    };
+
+    typedef BOOST_DEDUCED_TYPENAME mpl::eval_if<
+            // if its primitive
+            mpl::equal_to<
+                boost::serialization::implementation_level<T>,
+                mpl::int_<boost::serialization::primitive_type>
+            >,
+            mpl::identity<load_primitive>,
+        // else
+        BOOST_DEDUCED_TYPENAME mpl::eval_if<
+        // class info / version
+        mpl::greater_equal<
+                    boost::serialization::implementation_level<T>,
+                    mpl::int_<boost::serialization::object_class_info>
+                >,
+        // do standard load
+        mpl::identity<load_standard>,
+    // else
+    BOOST_DEDUCED_TYPENAME mpl::eval_if<
+        // no tracking
+                mpl::equal_to<
+                    boost::serialization::tracking_level<T>,
+                    mpl::int_<boost::serialization::track_never>
+            >,
+            // do a fast load
+            mpl::identity<load_only>,
+        // else
+        // do a fast load only tracking is turned off
+        mpl::identity<load_conditional>
+    > > >::type typex;
+
     static void invoke(Archive & ar, T &t){
+        // check that we're not trying to serialize something that
+        // has been marked not to be serialized.  If this your program
+        // traps here, you've tried to serialize a class whose trait
+        // has been marked "non-serializable". Either reset the trait
+        // (see level.hpp) or change program not to serialize items of this class
         BOOST_STATIC_ASSERT((
             mpl::greater_equal<
                 boost::serialization::implementation_level<T>, 
                 mpl::int_<boost::serialization::primitive_type>
             >::value
         ));
-        typedef BOOST_DEDUCED_TYPENAME mpl::eval_if<
-                // if its primitive
-                mpl::equal_to<
-                    boost::serialization::implementation_level<T>,
-                    mpl::int_<boost::serialization::primitive_type>
-                >,
-                mpl::identity<load_primitive>,
-            // else
-            BOOST_DEDUCED_TYPENAME mpl::eval_if<
-                mpl::and_<
-                    // no class info / version
-                    mpl::less<
-                        boost::serialization::implementation_level<T>,
-                        mpl::int_<boost::serialization::object_class_info>
-                    >,
-                    // and no tracking
-                    mpl::equal_to<
-                        boost::serialization::tracking_level<T>,
-                        mpl::int_<boost::serialization::track_never>
-                    >
-                >,
-                // do a fast load
-                mpl::identity<load_only>,
-            // else
-                // do standard load
-                mpl::identity<load>
-            >
-        >::type typex;
         typex::invoke(ar, t);
     }
 };
@@ -373,10 +398,8 @@ struct load_pointer_type {
     struct abstract
     {
         static const basic_pointer_iserializer * register_type(Archive & /* ar */){
-            typedef BOOST_DEDUCED_TYPENAME 
-                boost::serialization::type_info_implementation<T>::type::is_polymorphic typex;
             // it has? to be polymorphic
-            BOOST_STATIC_ASSERT(typex::value);
+            BOOST_STATIC_ASSERT(boost::is_polymorphic<T>::value);
             return static_cast<basic_pointer_iserializer *>(NULL);
          }
     };
@@ -391,38 +414,13 @@ struct load_pointer_type {
 
     template<class T>
     static const basic_pointer_iserializer * register_type(Archive &ar, T & /*t*/){
-        #if defined(BOOST_MSVC)
-        // note: if you program traps here its because
-        // a) your serializing through a base class pointer
-        // b) to an archive not in the known list.  
-        // This will usually occur when one makes a custom archive and 
-        // forgets to add it to the list of known archive.  If the derived
-        // class is explictly registered or if no derived pointer is used
-        // there won't be a problem - that's why its a warning.  However
-        // if you export the derived type and the archive used isn't on the
-        // known list it will fail below at execution time and one will have
-        // a hell of time figuring out why.  Hence this warning.
-        BOOST_STATIC_WARNING((
-            mpl::not_<
-                mpl::and_<
-                    BOOST_DEDUCED_TYPENAME serialization::type_info_implementation<T>::type::is_polymorphic,
-                    mpl::not_<mpl::empty<known_archive_types<false>::type > >,
-                    is_same<
-                        mpl::end<known_archive_types<false>::type >::type,
-                        BOOST_DEDUCED_TYPENAME mpl::find<known_archive_types<false>::type, Archive>::type
-                    >
-                >
-            >
-            ::value
-        ));
-        #endif
         // there should never be any need to load an abstract polymorphic 
         // class pointer.  Inhibiting code generation for this
         // permits abstract base classes to be used - note: exception
         // virtual serialize functions used for plug-ins
         typedef BOOST_DEDUCED_TYPENAME
             mpl::eval_if<
-                is_abstract<T>,
+                boost::serialization::is_abstract<const T>,
                 mpl::identity<abstract<T> >,
                 mpl::identity<non_abstract<T> >    
             >::type typex;
@@ -439,7 +437,8 @@ struct load_pointer_type {
         return static_cast<T *>(
             boost::serialization::void_upcast(
                 eti,
-                * boost::serialization::type_info_implementation<T>::type::get_instance(),
+                boost::serialization::type_info_implementation<T>::type
+                    ::get_const_instance(),
                 t
             )
         );
@@ -454,7 +453,7 @@ struct load_pointer_type {
         );
         // if the pointer isn't that of the base class
         if(newbpis_ptr != bpis_ptr){
-            t = pointer_tweak(newbpis_ptr->type, t, *t);
+            t = pointer_tweak(newbpis_ptr->get_eti(), t, *t);
         }
     }
 };
@@ -472,6 +471,8 @@ struct load_enum_type {
 template<class Archive, class T>
 struct load_array_type {
     static void invoke(Archive &ar, T &t){
+        typedef BOOST_DEDUCED_TYPENAME remove_extent<T>::type value_type;
+        
         // convert integers to correct enum to load
         int current_count = sizeof(t) / (
             static_cast<char *>(static_cast<void *>(&t[1])) 
@@ -480,38 +481,22 @@ struct load_array_type {
         int count;
         ar >> BOOST_SERIALIZATION_NVP(count);
         if(count > current_count)
-            boost::throw_exception(archive::archive_exception(
+            boost::serialization::throw_exception(archive::archive_exception(
                 boost::archive::archive_exception::array_size_too_short
             ));
-        int i;
-        for(i = 0; i < count; ++i)
-            ar >> boost::serialization::make_nvp("item", t[i]);
+        ar >> serialization::make_array(static_cast<value_type*>(&t[0]),count);
     }
 };
 
-// note bogus arguments to workaround msvc 6 silent runtime failure
-template<class Archive, class T>
-BOOST_DLLEXPORT inline const basic_pointer_iserializer &
-instantiate_pointer_iserializer(
-    Archive * /* ar = NULL */,
-    T * /* t = NULL */
-) BOOST_USED;
-
-template<class Archive, class T>
-BOOST_DLLEXPORT inline const basic_pointer_iserializer &
-instantiate_pointer_iserializer(
-    Archive * /* ar = NULL */,
-    T * /* t = NULL */
-){
-    // note: reversal of order of arguments to work around msvc 6.0 bug
-    // that manifests itself while trying to link.
-    return pointer_iserializer<T, Archive>::instance;
-}
-
 } // detail
 
 template<class Archive, class T>
 inline void load(Archive &ar, T &t){
+    // if this assertion trips. It means we're trying to load a
+    // const object with a compiler that doesn't have correct
+    // funtion template ordering.  On other compilers, this is
+    // handled below.
+    BOOST_STATIC_ASSERT(! boost::is_const<T>::value);
     typedef
         BOOST_DEDUCED_TYPENAME mpl::eval_if<is_pointer<T>,
             mpl::identity<detail::load_pointer_type<Archive, T> >
@@ -529,6 +514,36 @@ inline void load(Archive &ar, T &t){
     typex::invoke(ar, t);
 }
 
+// BORLAND
+#if BOOST_WORKAROUND(__BORLANDC__, BOOST_TESTED_AT(0x560))
+// borland has a couple of problems
+// a) if function is partiall specialized - see below
+// const paramters are transformed to non-const ones
+// b) implementation of base_object can't be made to work
+// correctly which results in all base_object s being const.
+// So, strip off the const for borland.  This breaks the trap
+// for loading const objects - but I see no alternative
+template<class Archive, class T>
+inline void load(Archive &ar, const T & t){
+    load(ar, const_cast<T &>(t));
+}
+#endif
+
+// let wrappers through.
+#ifndef BOOST_NO_FUNCTION_TEMPLATE_ORDERING
+template<class Archive, class T>
+inline void load_wrapper(Archive &ar, const T&t, mpl::true_){
+    boost::archive::load(ar, const_cast<T&>(t));
+}
+
+#if !BOOST_WORKAROUND(__BORLANDC__, BOOST_TESTED_AT(0x560))
+template<class Archive, class T>
+inline void load(Archive &ar, const T&t){
+  load_wrapper(ar,t,serialization::is_wrapper<T>());
+}
+#endif 
+#endif
+
 } // namespace archive
 } // namespace boost
 
