@@ -21,6 +21,7 @@
 #include <boost/config.hpp>
 #include <boost/asio/detail/pop_options.hpp>
 
+#include <boost/asio/detail/eventfd_select_interrupter.hpp>
 #include <boost/asio/detail/pipe_select_interrupter.hpp>
 #include <boost/asio/detail/socket_select_interrupter.hpp>
 
@@ -30,6 +31,8 @@ namespace detail {
 
 #if defined(BOOST_WINDOWS) || defined(__CYGWIN__)
 typedef socket_select_interrupter select_interrupter;
+#elif defined(BOOST_ASIO_HAS_EVENTFD)
+typedef eventfd_select_interrupter select_interrupter;
 #else
 typedef pipe_select_interrupter select_interrupter;
 #endif
