@@ -1,5 +1,5 @@
 /*=============================================================================
-    Copyright (c) 2001-2008 Hartmut Kaiser
+    Copyright (c) 2001-2009 Hartmut Kaiser
 
     Distributed under the Boost Software License, Version 1.0. (See accompanying 
     file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -491,69 +491,69 @@ namespace boost { namespace spirit { namespace char_class
         /*    189  bd */   '\0',
         /*    190  be */   '\0',
         /*    191  bf */   '\0',
-        /*    192  c0 */   0xc0,
-        /*    193  c1 */   0xc1,
-        /*    194  c2 */   0xc2,
-        /*    195  c3 */   0xc3,
-        /*    196  c4 */   0xc4,
-        /*    197  c5 */   0xc5,
-        /*    198  c6 */   0xc6,
-        /*    199  c7 */   0xc7,
-        /*    200  c8 */   0xc8,
-        /*    201  c9 */   0xc9,
-        /*    202  ca */   0xca,
-        /*    203  cb */   0xcb,
-        /*    204  cc */   0xcc,
-        /*    205  cd */   0xcd,
-        /*    206  ce */   0xce,
-        /*    207  cf */   0xcf,
-        /*    208  d0 */   0xd0,
-        /*    209  d1 */   0xd1,
-        /*    210  d2 */   0xd2,
-        /*    211  d3 */   0xd3,
-        /*    212  d4 */   0xd4,
-        /*    213  d5 */   0xd5,
-        /*    214  d6 */   0xd6,
+        /*    192  c0 */   0xe0,
+        /*    193  c1 */   0xe1,
+        /*    194  c2 */   0xe2,
+        /*    195  c3 */   0xe3,
+        /*    196  c4 */   0xe4,
+        /*    197  c5 */   0xe5,
+        /*    198  c6 */   0xe6,
+        /*    199  c7 */   0xe7,
+        /*    200  c8 */   0xe8,
+        /*    201  c9 */   0xe9,
+        /*    202  ca */   0xea,
+        /*    203  cb */   0xeb,
+        /*    204  cc */   0xec,
+        /*    205  cd */   0xed,
+        /*    206  ce */   0xee,
+        /*    207  cf */   0xef,
+        /*    208  d0 */   0xf0,
+        /*    209  d1 */   0xf1,
+        /*    210  d2 */   0xf2,
+        /*    211  d3 */   0xf3,
+        /*    212  d4 */   0xf4,
+        /*    213  d5 */   0xf5,
+        /*    214  d6 */   0xf6,
         /*    215  d7 */   '\0',
-        /*    216  d8 */   0xd8,
-        /*    217  d9 */   0xd9,
-        /*    218  da */   0xda,
-        /*    219  db */   0xdb,
-        /*    220  dc */   0xdc,
-        /*    221  dd */   0xdd,
-        /*    222  de */   0xde,
+        /*    216  d8 */   0xf8,
+        /*    217  d9 */   0xf9,
+        /*    218  da */   0xfa,
+        /*    219  db */   0xfb,
+        /*    220  dc */   0xfc,
+        /*    221  dd */   0xfd,
+        /*    222  de */   0xfe,
         /*    223  df */   '\0',
-        /*    224  e0 */   0xe0,
-        /*    225  e1 */   0xe1,
-        /*    226  e2 */   0xe2,
-        /*    227  e3 */   0xe3,
-        /*    228  e4 */   0xe4,
-        /*    229  e5 */   0xe5,
-        /*    230  e6 */   0xe6,
-        /*    231  e7 */   0xe7,
-        /*    232  e8 */   0xe8,
-        /*    233  e9 */   0xe9,
-        /*    234  ea */   0xea,
-        /*    235  eb */   0xeb,
-        /*    236  ec */   0xec,
-        /*    237  ed */   0xed,
-        /*    238  ee */   0xee,
-        /*    239  ef */   0xef,
-        /*    240  f0 */   0xf0,
-        /*    241  f1 */   0xf1,
-        /*    242  f2 */   0xf2,
-        /*    243  f3 */   0xf3,
-        /*    244  f4 */   0xf4,
-        /*    245  f5 */   0xf5,
-        /*    246  f6 */   0xf6,
+        /*    224  e0 */   0xc0,
+        /*    225  e1 */   0xc1,
+        /*    226  e2 */   0xc2,
+        /*    227  e3 */   0xc3,
+        /*    228  e4 */   0xc4,
+        /*    229  e5 */   0xc5,
+        /*    230  e6 */   0xc6,
+        /*    231  e7 */   0xc7,
+        /*    232  e8 */   0xc8,
+        /*    233  e9 */   0xc9,
+        /*    234  ea */   0xca,
+        /*    235  eb */   0xcb,
+        /*    236  ec */   0xcc,
+        /*    237  ed */   0xcd,
+        /*    238  ee */   0xce,
+        /*    239  ef */   0xcf,
+        /*    240  f0 */   0xd0,
+        /*    241  f1 */   0xd1,
+        /*    242  f2 */   0xd2,
+        /*    243  f3 */   0xd3,
+        /*    244  f4 */   0xd4,
+        /*    245  f5 */   0xd5,
+        /*    246  f6 */   0xd6,
         /*    247  f7 */   '\0',
-        /*    248  f8 */   0xf8,
-        /*    249  f9 */   0xf9,
-        /*    250  fa */   0xfa,
-        /*    251  fb */   0xfb,
-        /*    252  fc */   0xfc,
-        /*    253  fd */   0xfd,
-        /*    254  fe */   0xfe,
+        /*    248  f8 */   0xd8,
+        /*    249  f9 */   0xd9,
+        /*    250  fa */   0xda,
+        /*    251  fb */   0xdb,
+        /*    252  fc */   0xdc,
+        /*    253  fd */   0xdd,
+        /*    254  fe */   0xde,
         /*    255  ff */   '\0',
     };
 
@@ -562,7 +562,7 @@ namespace boost { namespace spirit { namespace char_class
     ///////////////////////////////////////////////////////////////////////////
     struct iso8859_1
     {
-        typedef char char_type;
+        typedef unsigned char char_type;
 
         static bool
         is_ascii(int ch)
