@@ -11,21 +11,13 @@
 #ifndef BOOST_UNITS_CMATH_HPP 
 #define BOOST_UNITS_CMATH_HPP
 
-#include <boost/version.hpp>
-
-// The trunk still has BOOST_VERSION == 103500
-//#if (BOOST_VERSION <= 103500)
-#if 0
-    #include <boost/units/detail/cmath_boost_1_35.hpp>
-#else
-
-#include <cmath>
+#include <boost/config/no_tr1/cmath.hpp>
 #include <cstdlib>
 
 #include <boost/math/special_functions/fpclassify.hpp>
 #include <boost/math/special_functions/hypot.hpp>
-//#include <boost/math/special_functions/next.hpp>
-//#include <boost/math/special_functions/round.hpp>
+#include <boost/math/special_functions/next.hpp>
+#include <boost/math/special_functions/round.hpp>
 #include <boost/math/special_functions/sign.hpp>
 
 #include <boost/units/dimensionless_quantity.hpp>
@@ -349,9 +341,6 @@ nearbyint BOOST_PREVENT_MACRO_SUBSTITUTION (const quantity<Unit,Y>& q)
 
 #endif
 
-// these are not in the release branch yet
-#if 0
-
 template<class Unit,class Y>
 inline 
 quantity<Unit,Y> nextafter BOOST_PREVENT_MACRO_SUBSTITUTION (const quantity<Unit,Y>& q1,
@@ -378,8 +367,6 @@ quantity<Unit,Y> nexttoward BOOST_PREVENT_MACRO_SUBSTITUTION (const quantity<Uni
     return quantity_type::from_value(nextafter BOOST_PREVENT_MACRO_SUBSTITUTION (q1.value(),q2.value()));
 }
 
-#endif
-
 #if 0
 
 template<class Unit,class Y>
@@ -394,6 +381,8 @@ rint BOOST_PREVENT_MACRO_SUBSTITUTION (const quantity<Unit,Y>& q)
     return quantity_type::from_value(rint BOOST_PREVENT_MACRO_SUBSTITUTION (q.value()));
 }
 
+#endif
+
 template<class Unit,class Y>
 inline 
 quantity<Unit,Y> 
@@ -406,8 +395,6 @@ round BOOST_PREVENT_MACRO_SUBSTITUTION (const quantity<Unit,Y>& q)
     return quantity_type::from_value(round BOOST_PREVENT_MACRO_SUBSTITUTION (q.value()));
 }
 
-#endif
-
 template<class Unit,class Y>
 inline 
 int 
@@ -551,8 +538,6 @@ sqrt(const quantity<Unit,Y>& q)
 
 } // namespace boost
 
-#endif // (BOOST_VERSION <= 103500)
-
 namespace boost {
 
 namespace units {
