@@ -3,7 +3,7 @@
 
     http://www.boost.org/
 
-    Copyright (c) 2001-2005 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -11,22 +11,21 @@
 #if !defined(CPP_EXPRESSION_GRAMMAR_HPP_099CD1A4_A6C0_44BE_8F24_0B00F5BE5674_INCLUDED)
 #define CPP_EXPRESSION_GRAMMAR_HPP_099CD1A4_A6C0_44BE_8F24_0B00F5BE5674_INCLUDED
 
-#include <boost/assert.hpp>
-#include <boost/spirit/core.hpp>
-#include <boost/spirit/attribute/closure.hpp>
-#include <boost/spirit/dynamic/if.hpp>
-#if SPIRIT_VERSION >= 0x1700
-#include <boost/spirit/actor/assign_actor.hpp>
-#include <boost/spirit/actor/push_back_actor.hpp>
-#endif // SPIRIT_VERSION >= 0x1700
-
-#include <boost/spirit/phoenix/functions.hpp>
-#include <boost/spirit/phoenix/operators.hpp>
-#include <boost/spirit/phoenix/primitives.hpp>
-#include <boost/spirit/phoenix/statements.hpp>
-#include <boost/spirit/phoenix/casts.hpp>
-
 #include <boost/wave/wave_config.hpp>
+
+#include <boost/assert.hpp>
+#include <boost/spirit/include/classic_core.hpp>
+#include <boost/spirit/include/classic_closure.hpp>
+#include <boost/spirit/include/classic_if.hpp>
+#include <boost/spirit/include/classic_assign_actor.hpp>
+#include <boost/spirit/include/classic_push_back_actor.hpp>
+
+#include <boost/spirit/include/phoenix1_functions.hpp>
+#include <boost/spirit/include/phoenix1_operators.hpp>
+#include <boost/spirit/include/phoenix1_primitives.hpp>
+#include <boost/spirit/include/phoenix1_statements.hpp>
+#include <boost/spirit/include/phoenix1_casts.hpp>
+
 #include <boost/wave/token_ids.hpp>
 
 #include <boost/wave/cpp_exceptions.hpp>
@@ -37,15 +36,15 @@
 #include <boost/wave/util/macro_helpers.hpp>
 
 #if !defined(spirit_append_actor)
-#if SPIRIT_VERSION >= 0x1700
-#define spirit_append_actor(actor) boost::spirit::push_back_a(actor)
-#define spirit_assign_actor(actor) boost::spirit::assign_a(actor)
-#else
-#define spirit_append_actor(actor) boost::spirit::append(actor)
-#define spirit_assign_actor(actor) boost::spirit::assign(actor)
-#endif // SPIRIT_VERSION >= 0x1700
+#define spirit_append_actor(actor) boost::spirit::classic::push_back_a(actor)
+#define spirit_assign_actor(actor) boost::spirit::classic::assign_a(actor)
 #endif // !defined(spirit_append_actor)
 
+// this must occur after all of the includes and before any code appears
+#ifdef BOOST_HAS_ABI_HEADERS
+#include BOOST_ABI_PREFIX
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  Encapsulation of the grammar for evaluation of constant preprocessor
@@ -67,7 +66,7 @@ namespace closures {
 //
 ///////////////////////////////////////////////////////////////////////////////
     struct cpp_expr_closure 
-    :   boost::spirit::closure<cpp_expr_closure, closure_value> 
+    :   boost::spirit::classic::closure<cpp_expr_closure, closure_value> 
     {
         member1 val;
     };
@@ -78,7 +77,7 @@ namespace impl {
 
 ///////////////////////////////////////////////////////////////////////////////
 //
-//  convert the given tokenvalue (integer literal) to a unsigned long
+//  convert the given token value (integer literal) to a unsigned long
 //
 ///////////////////////////////////////////////////////////////////////////////
     struct convert_intlit {
@@ -93,19 +92,20 @@ namespace impl {
         boost::wave::grammars::closures::closure_value 
         operator()(TokenT const &token) const
         { 
-            typedef boost::wave::grammars::closures::closure_value return_t;
+            typedef boost::wave::grammars::closures::closure_value return_type;
             bool is_unsigned = false;
-            unsigned long ul = intlit_grammar_gen<TokenT>::evaluate(token, 
+            uint_literal_type ul = intlit_grammar_gen<TokenT>::evaluate(token, 
                 is_unsigned);
 
-            return is_unsigned ? return_t(ul) : return_t(static_cast<long>(ul));
+            return is_unsigned ? 
+                return_type(ul) : return_type(static_cast<int_literal_type>(ul));
         }
     };
     phoenix::function<convert_intlit> const as_intlit;
 
 ///////////////////////////////////////////////////////////////////////////////
 //
-//  convert the given tokenvalue (character literal) to a unsigned int
+//  convert the given token value (character literal) to a unsigned int
 //
 ///////////////////////////////////////////////////////////////////////////////
     struct convert_chlit {
@@ -120,15 +120,17 @@ namespace impl {
         boost::wave::grammars::closures::closure_value 
         operator()(TokenT const &token) const
         { 
-            typedef boost::wave::grammars::closures::closure_value return_t;
-            return return_t(chlit_grammar_gen<TokenT>::evaluate(token));
+            typedef boost::wave::grammars::closures::closure_value return_type;
+            value_error status = error_noerror;
+            unsigned int value = chlit_grammar_gen<TokenT>::evaluate(token, status);
+            return return_type(value, status);
         }
     };
     phoenix::function<convert_chlit> const as_chlit;
 
 ////////////////////////////////////////////////////////////////////////////////
 //
-//  Handle the ?: operator with correct type propagation
+//  Handle the ?: operator with correct type and error propagation
 //
 ////////////////////////////////////////////////////////////////////////////////
     struct operator_questionmark {
@@ -143,12 +145,121 @@ namespace impl {
         boost::wave::grammars::closures::closure_value 
         operator()(CondT const &cond, Arg1T &val1, Arg2T const &val2) const
         { 
-            typedef boost::wave::grammars::closures::closure_value return_t;
-            return return_t(val1.handle_questionmark(cond, val2));
+            return val1.handle_questionmark(cond, val2);
         }
     };
     phoenix::function<operator_questionmark> const questionmark;
+
+///////////////////////////////////////////////////////////////////////////////
+//
+//  Handle type conversion conserving error conditions
+//
+///////////////////////////////////////////////////////////////////////////////
+    struct operator_to_bool {
     
+        template <typename ArgT>
+        struct result { 
+        
+            typedef boost::wave::grammars::closures::closure_value type; 
+        };
+
+        template <typename ArgT>
+        boost::wave::grammars::closures::closure_value 
+        operator()(ArgT &val) const
+        { 
+            typedef boost::wave::grammars::closures::closure_value return_type;
+            return return_type(
+                boost::wave::grammars::closures::as_bool(val), val.is_valid());
+        }
+    };
+    phoenix::function<operator_to_bool> const to_bool;
+    
+///////////////////////////////////////////////////////////////////////////////
+//
+//  Handle explicit type conversion
+//
+///////////////////////////////////////////////////////////////////////////////
+    struct operator_as_bool {
+    
+        template <typename ArgT>
+        struct result { 
+        
+            typedef bool type; 
+        };
+
+        template <typename ArgT>
+        bool
+        operator()(ArgT &val) const
+        { 
+            return boost::wave::grammars::closures::as_bool(val);
+        }
+    };
+    phoenix::function<operator_as_bool> const as_bool;
+    
+///////////////////////////////////////////////////////////////////////////////
+//
+//  Handle closure value operators with proper error propagation
+//
+///////////////////////////////////////////////////////////////////////////////
+#define BOOST_WAVE_BINARYOP(op, optok)                                        \
+    struct operator_binary_ ## op {                                           \
+                                                                              \
+        template <typename Arg1T, typename Arg2T>                             \
+        struct result {                                                       \
+                                                                              \
+            typedef boost::wave::grammars::closures::closure_value type;      \
+        };                                                                    \
+                                                                              \
+        template <typename Arg1T, typename Arg2T>                             \
+        boost::wave::grammars::closures::closure_value                        \
+        operator()(Arg1T &val1, Arg2T &val2) const                            \
+        {                                                                     \
+            return val1 optok val2;                                           \
+        }                                                                     \
+    };                                                                        \
+    phoenix::function<operator_binary_ ## op> const binary_ ## op             \
+    /**/
+
+    BOOST_WAVE_BINARYOP(and, &&);
+    BOOST_WAVE_BINARYOP(or, ||);
+    
+    BOOST_WAVE_BINARYOP(bitand, &);
+    BOOST_WAVE_BINARYOP(bitor, |);
+    BOOST_WAVE_BINARYOP(bitxor, ^);
+    
+    BOOST_WAVE_BINARYOP(lesseq, <=);
+    BOOST_WAVE_BINARYOP(less, <);
+    BOOST_WAVE_BINARYOP(greater, >);
+    BOOST_WAVE_BINARYOP(greateq, >=);
+    BOOST_WAVE_BINARYOP(eq, ==);
+    BOOST_WAVE_BINARYOP(ne, !=);
+
+#undef BOOST_WAVE_BINARYOP
+
+///////////////////////////////////////////////////////////////////////////////
+#define BOOST_WAVE_UNARYOP(op, optok)                                         \
+    struct operator_unary_ ## op {                                            \
+                                                                              \
+        template <typename ArgT>                                              \
+        struct result {                                                       \
+                                                                              \
+            typedef boost::wave::grammars::closures::closure_value type;      \
+        };                                                                    \
+                                                                              \
+        template <typename ArgT>                                              \
+        boost::wave::grammars::closures::closure_value                        \
+        operator()(ArgT &val) const                                           \
+        {                                                                     \
+            return optok val;                                                 \
+        }                                                                     \
+    };                                                                        \
+    phoenix::function<operator_unary_ ## op> const unary_ ## op               \
+    /**/
+
+    BOOST_WAVE_UNARYOP(neg, !);
+    
+#undef BOOST_WAVE_UNARYOP
+
 }   // namespace impl
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -158,7 +269,7 @@ namespace impl {
     /**/
 
 struct expression_grammar :
-    public boost::spirit::grammar<
+    public boost::spirit::classic::grammar<
         expression_grammar, 
         closures::cpp_expr_closure::context_t
     >
@@ -169,12 +280,16 @@ struct expression_grammar :
             TRACE_CPP_EXPR_GRAMMAR);
     }
     
+    // no need for copy constructor/assignment operator
+    expression_grammar(expression_grammar const&);
+    expression_grammar& operator= (expression_grammar const&);
+    
     template <typename ScannerT>
     struct definition
     {
         typedef closures::cpp_expr_closure closure_type;
-        typedef boost::spirit::rule<ScannerT, closure_type::context_t> rule_t;
-        typedef boost::spirit::rule<ScannerT> simple_rule_t;
+        typedef boost::spirit::classic::rule<ScannerT, closure_type::context_t> rule_t;
+        typedef boost::spirit::classic::rule<ScannerT> simple_rule_t;
 
         simple_rule_t pp_expression;
         
@@ -194,11 +309,11 @@ struct expression_grammar :
         rule_t add_exp_nocalc, multiply_exp_nocalc;
         rule_t unary_exp_nocalc, primary_exp_nocalc, constant_nocalc;
 
-        boost::spirit::subrule<0, closure_type::context_t> const_exp_subrule;
+        boost::spirit::classic::subrule<0, closure_type::context_t> const_exp_subrule;
 
         definition(expression_grammar const &self)
         {
-            using namespace boost::spirit;
+            using namespace boost::spirit::classic;
             using namespace phoenix;
             using namespace boost::wave;
             using boost::wave::util::pattern_p;
@@ -227,7 +342,7 @@ struct expression_grammar :
 
             logical_or_exp 
                 =   logical_and_exp[logical_or_exp.val = arg1]
-                    >> *(   if_p(static_cast_<bool>(logical_or_exp.val))
+                    >> *(   if_p(impl::as_bool(logical_or_exp.val))
                             [
                                 // if one of the || operators is true, no more
                                 // evaluation is required
@@ -235,7 +350,7 @@ struct expression_grammar :
                                 >>  logical_and_exp_nocalc
                                     [
                                         logical_or_exp.val = 
-                                            static_cast_<bool>(logical_or_exp.val)
+                                            impl::to_bool(logical_or_exp.val)
                                     ]
                             ]
                             .else_p
@@ -244,7 +359,7 @@ struct expression_grammar :
                                 >>  logical_and_exp
                                     [
                                         logical_or_exp.val = 
-                                            logical_or_exp.val || arg1
+                                            impl::binary_or(logical_or_exp.val, arg1)
                                     ]
                             ]
                         )
@@ -252,13 +367,13 @@ struct expression_grammar :
 
             logical_and_exp
                 =   inclusive_or_exp[logical_and_exp.val = arg1]
-                    >> *(   if_p(static_cast_<bool>(logical_and_exp.val))
+                    >> *(   if_p(impl::as_bool(logical_and_exp.val))
                             [
                                 pattern_p(T_ANDAND, MainTokenMask)
                                 >>  inclusive_or_exp
                                     [
                                         logical_and_exp.val = 
-                                            logical_and_exp.val && arg1
+                                            impl::binary_and(logical_and_exp.val, arg1)
                                     ]
                             ]
                             .else_p
@@ -269,7 +384,7 @@ struct expression_grammar :
                                 >>  inclusive_or_exp_nocalc
                                     [
                                         logical_and_exp.val =
-                                            static_cast_<bool>(logical_and_exp.val)
+                                            impl::to_bool(logical_and_exp.val)
                                     ]
                             ]
                         )
@@ -281,8 +396,7 @@ struct expression_grammar :
                             >>  exclusive_or_exp
                                 [
                                     inclusive_or_exp.val = 
-                                          static_cast_<unsigned int>(inclusive_or_exp.val) 
-                                      |   static_cast_<unsigned int>(arg1)
+                                        impl::binary_bitor(inclusive_or_exp.val, arg1)
                                 ]
                         )
                 ;
@@ -293,8 +407,7 @@ struct expression_grammar :
                             >>  and_exp
                                 [
                                     exclusive_or_exp.val = 
-                                          static_cast_<unsigned int>(exclusive_or_exp.val)
-                                      ^   static_cast_<unsigned int>(arg1)
+                                        impl::binary_bitxor(exclusive_or_exp.val, arg1)
                                 ]
                         )
                 ;
@@ -305,8 +418,7 @@ struct expression_grammar :
                             >>  cmp_equality
                                 [
                                     and_exp.val = 
-                                          static_cast_<unsigned int>(and_exp.val)
-                                      &   static_cast_<unsigned int>(arg1)
+                                        impl::binary_bitand(and_exp.val, arg1)
                                 ]
                         )
                 ;
@@ -317,13 +429,13 @@ struct expression_grammar :
                             >>  cmp_relational
                                 [
                                     cmp_equality.val = 
-                                        cmp_equality.val == arg1
+                                        impl::binary_eq(cmp_equality.val, arg1)
                                 ]
                         |   pattern_p(T_NOTEQUAL, MainTokenMask)
                             >>  cmp_relational
                                 [
                                     cmp_equality.val = 
-                                        cmp_equality.val != arg1
+                                        impl::binary_ne(cmp_equality.val, arg1)
                                 ]
                         )
                 ;
@@ -334,25 +446,25 @@ struct expression_grammar :
                             >>  shift_exp
                                 [
                                     cmp_relational.val = 
-                                        cmp_relational.val <= arg1
+                                        impl::binary_lesseq(cmp_relational.val, arg1)
                                 ]
                         |   ch_p(T_GREATEREQUAL)
                             >>  shift_exp
                                 [
                                     cmp_relational.val = 
-                                        cmp_relational.val >= arg1
+                                        impl::binary_greateq(cmp_relational.val, arg1)
                                 ]
                         |   ch_p(T_LESS)
                             >>  shift_exp
                                 [
                                     cmp_relational.val = 
-                                        cmp_relational.val < arg1
+                                        impl::binary_less(cmp_relational.val, arg1)
                                 ]
                         |   ch_p(T_GREATER)
                             >>  shift_exp
                                 [
                                     cmp_relational.val = 
-                                        cmp_relational.val > arg1
+                                        impl::binary_greater(cmp_relational.val, arg1)
                                 ]
                         )
                 ;
@@ -423,7 +535,7 @@ struct expression_grammar :
                     ]
                 |   pattern_p(T_NOT, MainTokenMask) >> unary_exp
                     [
-                        unary_exp.val = !arg1
+                        unary_exp.val = impl::unary_neg(arg1)
                     ]
                 ;
 
@@ -435,7 +547,11 @@ struct expression_grammar :
                 ;
 
             constant
-                =   ch_p(T_INTLIT) 
+                =   ch_p(T_PP_NUMBER) 
+                    [
+                        constant.val = impl::as_intlit(arg1)
+                    ]
+                |   ch_p(T_INTLIT) 
                     [
                         constant.val = impl::as_intlit(arg1)
                     ]
@@ -558,7 +674,8 @@ struct expression_grammar :
                 ;
 
             constant_nocalc
-                =   ch_p(T_INTLIT) 
+                =   ch_p(T_PP_NUMBER) 
+                |   ch_p(T_INTLIT) 
                 |   ch_p(T_CHARLIT) 
                 ;
 
@@ -624,9 +741,9 @@ expression_grammar_gen<TokenT>::evaluate(
     typename token_sequence_type::const_iterator const &first, 
     typename token_sequence_type::const_iterator const &last, 
     typename token_type::position_type const &act_pos,
-    bool if_block_status)
+    bool if_block_status, value_error &status)
 {
-    using namespace boost::spirit;
+    using namespace boost::spirit::classic;
     using namespace boost::wave;
     using namespace boost::wave::grammars::closures;
     
@@ -638,7 +755,10 @@ expression_grammar_gen<TokenT>::evaluate(
     parse_info<iterator_type> hit(first);
     closure_value result;             // expression result
     
-    try {
+#if !defined(BOOST_NO_EXCEPTIONS)
+    try 
+#endif
+    {
         expression_grammar g;             // expression grammar
         hit = parse (first, last, g[spirit_assign_actor(result)], 
                      ch_p(T_SPACE) | ch_p(T_CCOMMENT) | ch_p(T_CPPCOMMENT));
@@ -648,9 +768,10 @@ expression_grammar_gen<TokenT>::evaluate(
             if (if_block_status) {
                 string_type expression = as_string<string_type>(first, last);
                 if (0 == expression.size()) 
-                    expression = "empty expression";
+                    expression = "<empty expression>";
                 BOOST_WAVE_THROW(preprocess_exception, ill_formed_expression, 
                     expression.c_str(), act_pos);
+                return false;
             }
             else {
             //  as the if_block_status is false no errors will be reported
@@ -658,17 +779,20 @@ expression_grammar_gen<TokenT>::evaluate(
             }
         }
     }
-    catch (wave::preprocess_exception const& e) {
+#if !defined(BOOST_NO_EXCEPTIONS)
+    catch (boost::wave::preprocess_exception const& e) {
     // expression is illformed
         if (if_block_status) {
-            throw e;
+            boost::throw_exception(e);
+            return false;
         }
-        else {
+        else         {
         //  as the if_block_status is false no errors will be reported
             return false;
         }
     }
-        
+#endif
+
     if (!hit.full) {
     // The token list starts with a valid expression, but there remains 
     // something. If the remainder consists out of whitespace only, the 
@@ -685,16 +809,17 @@ expression_grammar_gen<TokenT>::evaluate(
             case T_NEWLINE:
             case T_EOF:
             case T_CPPCOMMENT:              // contains newline
-                return bool(result);        // expression is valid
+                return as_bool(result);     // expression is valid
                 
             default:
             // expression is illformed
                 if (if_block_status) {
                     string_type expression = as_string<string_type>(first, last);
                     if (0 == expression.size()) 
-                        expression = "empty expression";
+                        expression = "<empty expression>";
                     BOOST_WAVE_THROW(preprocess_exception, ill_formed_expression, 
                         expression.c_str(), act_pos);
+                    return false;
                 }
                 else {
                 //  as the if_block_status is false no errors will be reported
@@ -705,24 +830,11 @@ expression_grammar_gen<TokenT>::evaluate(
         }
     }
 
-    if (closure_value::error_noerror != result.is_valid()) {
-    // division by zero occured
-        string_type expression = as_string<string_type>(first, last);
-        if (0 == expression.size()) 
-            expression = "empty expression";
-            
-        if (closure_value::error_division_by_zero == result.is_valid()) {
-            BOOST_WAVE_THROW(preprocess_exception, division_by_zero, 
-                expression.c_str(), act_pos);
-        }
-        else if (closure_value::error_overflow == result.is_valid()) {
-            BOOST_WAVE_THROW(preprocess_exception, integer_overflow,
-                expression.c_str(), act_pos);
-        }
-    }
+    if (error_noerror != result.is_valid()) // division or other error by zero occurred
+        status = result.is_valid();
     
 // token sequence is a valid expression
-    return bool(result);
+    return as_bool(result);
 }
 
 #undef BOOST_WAVE_EXPRGRAMMAR_GEN_INLINE
@@ -732,4 +844,9 @@ expression_grammar_gen<TokenT>::evaluate(
 }   // namespace wave
 }   // namespace boost
 
+// the suffix header occurs after all of the code
+#ifdef BOOST_HAS_ABI_HEADERS
+#include BOOST_ABI_SUFFIX
+#endif
+
 #endif // !defined(CPP_EXPRESSION_GRAMMAR_HPP_099CD1A4_A6C0_44BE_8F24_0B00F5BE5674_INCLUDED)
