@@ -112,9 +112,9 @@ namespace
 {
   const error_code ok;
 
-  const fs::directory_iterator end_itr;
   bool is_empty_directory( const std::string & dir_path )
   {
+    static const fs::directory_iterator end_itr;
     return fs::directory_iterator(fs::path(dir_path)) == end_itr;
   }
 
@@ -133,9 +133,9 @@ namespace
   inline DWORD get_file_attributes( const wchar_t * ph )
     { return ::GetFileAttributesW( ph ); }
 
-  const fs::wdirectory_iterator wend_itr;
   bool is_empty_directory( const std::wstring & dir_path )
   {
+    static const fs::wdirectory_iterator wend_itr;
     return fs::wdirectory_iterator(fs::wpath(dir_path)) == wend_itr;
   }
 
@@ -583,9 +583,9 @@ namespace
         // rmdir() or unlink() as indicated.
         // Same bug also reported for QNX, with the same fix.
         int err = ::unlink( p );
-        if ( err != EPERM )
+        if ( err == 0 || errno != EPERM )
           return err;
-        return ::rmdir( p )
+        return ::rmdir( p );
 #     else
         return std::remove( p );
 #     endif
@@ -1205,7 +1205,7 @@ namespace boost
       {
         const std::size_t buf_sz = 32768;
         boost::scoped_array<char> buf( new char [buf_sz] );
-        int infile=0, outfile=0;  // init quiets compiler warning
+        int infile=-1, outfile=-1;  // -1 means not open
         struct stat from_stat;
 
         if ( ::stat( from_file_ph.c_str(), &from_stat ) != 0
