@@ -20,7 +20,7 @@ using namespace boost::interprocess;
 
 int main ()
 {
-   const int FileSize          = 65536;
+   const int FileSize          = 65536*10;
    const char *const FileName = test::get_process_id_name();
 
    //STL compatible allocator object for memory-mapped file
@@ -206,6 +206,8 @@ int main ()
          managed_mapped_file move_ctor(detail::move_impl(original));
          managed_mapped_file move_assign;
          move_assign = detail::move_impl(move_ctor);
+         move_assign.swap(detail::move_impl(original));
+         move_assign.swap(original);
       }
    }
 
