@@ -1,4 +1,4 @@
-//  (C) Copyright Gennadiy Rozental 2001-2005.
+//  (C) Copyright Gennadiy Rozental 2001-2006.
 //  Distributed under the Boost Software License, Version 1.0.
 //  (See accompanying file LICENSE_1_0.txt or copy at 
 //  http://www.boost.org/LICENSE_1_0.txt)
@@ -6,6 +6,10 @@
 //  See http://www.boost.org/libs/test for the library home page.
 
 // Boost.Test
+#ifdef BOOST_MSVC
+# pragma warning(disable: C4345)
+#endif
+
 #include <boost/test/test_tools.hpp>
 #include <boost/test/test_case_template.hpp>
 using boost::unit_test::test_suite;
