@@ -2,7 +2,7 @@
     Boost.Wave: A Standard compliant C++ preprocessor library
     http://www.boost.org/
 
-    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -13,12 +13,10 @@
     license reproduced at the end of this file.
 =============================================================================*/
 
-// Tests translation limits.
+// Tests error reporting: ill-formed group in a source file. 
 
-#undef NEST
-#define NEST 9
-
-#include "t_5_035_10.hpp"
+// 17.5: Errorneous #endif without #if in an included file.
+#endif
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
